@@ -18,17 +18,11 @@ pub(crate) enum CompileErrorKind<'src> {
     circle: Vec<&'src str>,
   },
   DependencyArgumentCountMismatch {
-    dependency: &'src str,
+    dependency: Namepath<'src>,
     found: usize,
     min: usize,
     max: usize,
   },
-  Redefinition {
-    first: usize,
-    first_type: &'static str,
-    name: &'src str,
-    second_type: &'static str,
-  },
   DuplicateAttribute {
     attribute: &'src str,
     first: usize,
@@ -41,10 +35,10 @@ pub(crate) enum CompileErrorKind<'src> {
     setting: &'src str,
     first: usize,
   },
-  DuplicateVariable {
+  DuplicateUnexport {
     variable: &'src str,
   },
-  DuplicateUnexport {
+  DuplicateVariable {
     variable: &'src str,
   },
   ExitMessageAndNoExitMessageAttribute {
@@ -97,6 +91,12 @@ pub(crate) enum CompileErrorKind<'src> {
     parameter: &'src str,
   },
   ParsingRecursionDepthExceeded,
+  Redefinition {
+    first: usize,
+    first_type: &'static str,
+    name: &'src str,
+    second_type: &'static str,
+  },
   RequiredParameterFollowsDefaultParameter {
     parameter: &'src str,
   },
@@ -138,14 +138,14 @@ pub(crate) enum CompileErrorKind<'src> {
   UnicodeEscapeUnterminated,
   UnknownAliasTarget {
     alias: &'src str,
-    target: &'src str,
+    target: Namepath<'src>,
   },
   UnknownAttribute {
     attribute: &'src str,
   },
   UnknownDependency {
     recipe: &'src str,
-    unknown: &'src str,
+    unknown: Namepath<'src>,
   },
   UnknownFunction {
     function: &'src str,
@@ -153,7 +153,9 @@ pub(crate) enum CompileErrorKind<'src> {
   UnknownSetting {
     setting: &'src str,
   },
-  UnknownStartOfToken,
+  UnknownStartOfToken {
+    start: char,
+  },
   UnpairedCarriageReturn,
   UnterminatedBacktick,
   UnterminatedInterpolation,
