@@ -77,19 +77,19 @@ fn nested_module_recipes_can_be_run_with_path_syntax() {
 fn invalid_path_syntax() {
   Test::new()
     .arg(":foo::foo")
-    .stderr("error: Justfile does not contain recipe `:foo::foo`.\n")
+    .stderr("error: Justfile does not contain recipe `:foo::foo`\n")
     .status(EXIT_FAILURE)
     .run();
 
   Test::new()
     .arg("foo::foo:")
-    .stderr("error: Justfile does not contain recipe `foo::foo:`.\n")
+    .stderr("error: Justfile does not contain recipe `foo::foo:`\n")
     .status(EXIT_FAILURE)
     .run();
 
   Test::new()
     .arg("foo:::foo")
-    .stderr("error: Justfile does not contain recipe `foo:::foo`.\n")
+    .stderr("error: Justfile does not contain recipe `foo:::foo`\n")
     .status(EXIT_FAILURE)
     .run();
 }
@@ -99,7 +99,7 @@ fn missing_recipe_after_invalid_path() {
   Test::new()
     .arg(":foo::foo")
     .arg("bar")
-    .stderr("error: Justfile does not contain recipe `:foo::foo`.\n")
+    .stderr("error: Justfile does not contain recipe `:foo::foo`\n")
     .status(EXIT_FAILURE)
     .run();
 }
@@ -655,7 +655,7 @@ fn submodule_recipe_not_found_spaced_error_message() {
       ",
     )
     .args(["foo", "baz"])
-    .stderr("error: Justfile does not contain recipe `foo baz`.\nDid you mean `bar`?\n")
+    .stderr("error: Justfile does not contain recipe `foo baz`\nDid you mean `bar`?\n")
     .status(1)
     .run();
 }
@@ -670,7 +670,7 @@ fn submodule_recipe_not_found_colon_separated_error_message() {
       ",
     )
     .args(["foo::baz"])
-    .stderr("error: Justfile does not contain recipe `foo::baz`.\nDid you mean `bar`?\n")
+    .stderr("error: Justfile does not contain recipe `foo::baz`\nDid you mean `bar`?\n")
     .status(1)
     .run();
 }
@@ -771,15 +771,209 @@ fn doc_attribute_on_module() {
     .run();
 }
 
+#[test]
+fn group_attribute_on_module() {
+  Test::new()
+    .write("foo.just", "")
+    .write("bar.just", "")
+    .write("zee.just", "")
+    .justfile(
+      r"
+        [group('alpha')]
+        mod zee
+
+        [group('alpha')]
+        mod foo
+
+        [group('alpha')]
+        a:
+
+        [group('beta')]
+        b:
+
+        [group('beta')]
+        mod bar
+
+        c:
+      ",
+    )
+    .test_round_trip(false)
+    .arg("--list")
+    .stdout(
+      "
+        Available recipes:
+            c
+
+            [alpha]
+            a
+            foo ...
+            zee ...
+
+            [beta]
+            b
+            bar ...
+      ",
+    )
+    .run();
+}
+
+#[test]
+fn group_attribute_on_module_unsorted() {
+  Test::new()
+    .write("foo.just", "")
+    .write("bar.just", "")
+    .write("zee.just", "")
+    .justfile(
+      r"
+        [group('alpha')]
+        mod zee
+
+        [group('alpha')]
+        mod foo
+
+        [group('alpha')]
+        a:
+
+        [group('beta')]
+        b:
+
+        [group('beta')]
+        mod bar
+
+        c:
+      ",
+    )
+    .test_round_trip(false)
+    .arg("--list")
+    .arg("--unsorted")
+    .stdout(
+      "
+        Available recipes:
+            c
+
+            [alpha]
+            a
+            zee ...
+            foo ...
+
+            [beta]
+            b
+            bar ...
+      ",
+    )
+    .run();
+}
+
+#[test]
+fn group_attribute_on_module_list_submodule() {
+  Test::new()
+    .write("foo.just", "d:")
+    .write("bar.just", "e:")
+    .write("zee.just", "f:")
+    .justfile(
+      r"
+        [group('alpha')]
+        mod zee
+
+        [group('alpha')]
+        mod foo
+
+        [group('alpha')]
+        a:
+
+        [group('beta')]
+        b:
+
+        [group('beta')]
+        mod bar
+
+        c:
+      ",
+    )
+    .test_round_trip(false)
+    .arg("--list")
+    .arg("--list-submodules")
+    .stdout(
+      "
+        Available recipes:
+            c
+
+            [alpha]
+            a
+            foo:
+                d
+            zee:
+                f
+
+            [beta]
+            b
+            bar:
+                e
+      ",
+    )
+    .run();
+}
+
+#[test]
+fn group_attribute_on_module_list_submodule_unsorted() {
+  Test::new()
+    .write("foo.just", "d:")
+    .write("bar.just", "e:")
+    .write("zee.just", "f:")
+    .justfile(
+      r"
+        [group('alpha')]
+        mod zee
+
+        [group('alpha')]
+        mod foo
+
+        [group('alpha')]
+        a:
+
+        [group('beta')]
+        b:
+
+        [group('beta')]
+        mod bar
+
+        c:
+      ",
+    )
+    .test_round_trip(false)
+    .arg("--list")
+    .arg("--list-submodules")
+    .arg("--unsorted")
+    .stdout(
+      "
+        Available recipes:
+            c
+
+            [alpha]
+            a
+            zee:
+                f
+            foo:
+                d
+
+            [beta]
+            b
+            bar:
+                e
+      ",
+    )
+    .run();
+}
+
 #[test]
 fn bad_module_attribute_fails() {
   Test::new()
     .write("foo.just", "")
     .justfile(
-      r#"
+      r"
         [no-cd]
         mod foo
-      "#,
+      ",
     )
     .test_round_trip(false)
     .arg("--list")
@@ -793,14 +987,85 @@ fn empty_doc_attribute_on_module() {
   Test::new()
     .write("foo.just", "")
     .justfile(
-      r#"
+      "
         # Suppressed comment
         [doc]
         mod foo
-      "#,
+      ",
     )
     .test_round_trip(false)
     .arg("--list")
     .stdout("Available recipes:\n    foo ...\n")
     .run();
 }
+
+#[test]
+fn overrides_work_when_submodule_is_present() {
+  Test::new()
+    .write("bar.just", "")
+    .justfile(
+      "
+        mod bar
+
+        x := 'a'
+
+        foo:
+          @echo {{ x }}
+      ",
+    )
+    .test_round_trip(false)
+    .arg("x=b")
+    .stdout("b\n")
+    .run();
+}
+
+#[test]
+fn exported_variables_are_available_in_submodules() {
+  Test::new()
+    .write("foo.just", "bar:\n @echo $x")
+    .justfile(
+      "
+        mod foo
+
+        export x := 'a'
+      ",
+    )
+    .test_round_trip(false)
+    .arg("foo::bar")
+    .stdout("a\n")
+    .run();
+}
+
+#[test]
+fn exported_variables_can_be_unexported_in_submodules() {
+  Test::new()
+    .write("foo.just", "unexport x\nbar:\n @echo ${x:-default}")
+    .justfile(
+      "
+        mod foo
+
+        export x := 'a'
+      ",
+    )
+    .test_round_trip(false)
+    .arg("foo::bar")
+    .stdout("default\n")
+    .run();
+}
+
+#[test]
+fn exported_variables_can_be_overridden_in_submodules() {
+  Test::new()
+    .write("foo.just", "export x := 'b'\nbar:\n @echo $x")
+    .justfile(
+      "
+        mod foo
+
+        export x := 'a'
+      ",
+    )
+    .test_round_trip(false)
+    .arg("foo::bar")
+    .stdout("b\n")
+    .run();
+}
